Medusa = Medusa or {}

Medusa.Sunspire = {
    name = "Sunspire",
    version = "1.1.0",
    combatStart = 0,
    debugData = {},
    boss = {},
    abilities = {},
    focusedFireStarted = 0,
    emberstormStarted = 0,
    fireStormStarted = 0,
    cataclysmStarted = 0,
    gravechillStarted = 0,
    timeShiftStarted = 0,
}

-- @TODO: Load default settings (normal?)
Medusa.Sunspire.default = {}

---------------------------------------------------
-- Inits Sunspire  --
---------------------------------------------------
function Medusa.InitSunspire()
    if Medusa.debug then
        d('MD: Sunspire loaded')
    end

    -- Exit, if no boss in range
    if not DoesUnitExist('boss1') then
        if Medusa.debug then
            d('MD: No boss detected, exiting')
        end
        return
    end

    -- Init boss detection module
    Medusa.Sunspire.initBossData()
    if Medusa.savedVariables.showVeteran and Medusa.Sunspire.boss.difficultyId < 2 then
        return
    end

    -- Init event for each boss
    Medusa.Sunspire.abilities = {}
    if Medusa.Sunspire.boss.id == '1' then
        if Medusa.debug then
            d('MD: Boss ' .. Medusa.Sunspire.boss.name .. ' engaged on ' .. Medusa.Sunspire.boss.difficultyName .. ' difficulty')
        end
        if Medusa.Sunspire.boss.difficultyId == 3 then
            Medusa.Sunspire.abilities = Medusa.Sunspire.YolnahkriinHM
        else
            Medusa.Sunspire.abilities = Medusa.Sunspire.Yolnahkriin
        end

        Medusa.Sunspire.StartCombatEvents('Yolnahkriin')
    elseif Medusa.Sunspire.boss.id == '2' then
        if Medusa.debug then
            d('MD: Boss ' .. Medusa.Sunspire.boss.name .. ' engaged on ' .. Medusa.Sunspire.boss.difficultyName .. ' difficulty')
        end
        if Medusa.Sunspire.boss.difficultyId == 3 then
            Medusa.Sunspire.abilities = Medusa.Sunspire.LokkestiizHM
        else
            Medusa.Sunspire.abilities = Medusa.Sunspire.Lokkestiiz
        end

        Medusa.Sunspire.StartCombatEvents('Lokkestiiz')
    elseif Medusa.Sunspire.boss.id == '3' then
        if Medusa.debug then
            d('MD: Boss ' .. Medusa.Sunspire.boss.name .. ' engaged on ' .. Medusa.Sunspire.boss.difficultyName .. ' difficulty')
        end
        if Medusa.Sunspire.boss.difficultyId == 3 then
            Medusa.Sunspire.abilities = Medusa.Sunspire.NahviintaasHM
        else
            Medusa.Sunspire.abilities = Medusa.Sunspire.Nahviintaas
        end

        Medusa.Sunspire.StartCombatEvents('Nahviintaas')
    else
        return
    end

    -- Init events
    Medusa.Sunspire.ActivateBars()
    Medusa.Sunspire.combatStart = tonumber(GetTimeStamp())
    EVENT_MANAGER:RegisterForEvent(Medusa.Sunspire.name, EVENT_COMBAT_EVENT, Medusa.SunspireCombatCallbacks)
end

---------------------------------------------------
-- Enable starting events  --
---------------------------------------------------
function Medusa.Sunspire.StartCombatEvents(bossName)
    if bossName == Medusa.Language.Settings_Sunspire_Yolnahkriin then
        if Medusa.savedVariables.focusedFireShow then
            Medusa.Sunspire.Events('First Focused Fire')
        end
        if Medusa.savedVariables.emberstormShow then
            Medusa.Sunspire.Events('First Emberstorm')
        end
    elseif bossName == Medusa.Language.Settings_Sunspire_Lokkestiiz then
        --        Medusa.Sunspire.Events('First Frozen Tomb')
    end
end

---------------------------------------------------
-- Activates bars  --
---------------------------------------------------
function Medusa.Sunspire.ActivateBars()
    -- Focused Fire
    Medusa.Sunspire.SetBarSize('FocusedFire', Medusa.savedVariables.focusedFireWidth, Medusa.savedVariables.focusedFireHeight)
    Medusa.Sunspire.SetBarColor('FocusedFire', unpack(Medusa.savedVariables.focusedFireColor))
    Medusa.Sunspire.SetBarPosition('FocusedFire', Medusa.savedVariables.focusedFireWindowOffsetX, Medusa.savedVariables.focusedFireWindowOffsetY)
    FocusedFireWindow:SetHidden(true)
    FocusedFireWindowLabel:SetText(Medusa.Language.Events_Sunspire_Next_FocusedFire_Soon)
    FocusedFireWindowStatusBar:SetMinMax(0, 1)
    FocusedFireWindowStatusBar:SetValue(1)

    -- Emberstorm
    Medusa.Sunspire.SetBarSize('Emberstorm', Medusa.savedVariables.emberstormWidth, Medusa.savedVariables.emberstormHeight)
    Medusa.Sunspire.SetBarColor('Emberstorm', unpack(Medusa.savedVariables.emberstormColor))
    Medusa.Sunspire.SetBarPosition('Emberstorm', Medusa.savedVariables.emberstormWindowOffsetX, Medusa.savedVariables.emberstormWindowOffsetY)
    EmberstormWindow:SetHidden(true)
    EmberstormWindowLabel:SetText(Medusa.Language.Events_Sunspire_Emberstorm)
    EmberstormWindowStatusBar:SetMinMax(0, 1)
    EmberstormWindowStatusBar:SetValue(1)

    -- Fire Storm
    Medusa.Sunspire.SetBarSize('FireStorm', Medusa.savedVariables.fireStormWidth, Medusa.savedVariables.fireStormHeight)
    Medusa.Sunspire.SetBarPosition('FireStorm', Medusa.savedVariables.fireStormWindowOffsetX, Medusa.savedVariables.fireStormWindowOffsetY)
    FireStormWindow:SetHidden(true)
    FireStormWindowLabel:SetText(Medusa.Language.Events_Sunspire_FireStorm)
    FireStormWindowFinishBar:SetMinMax(0, 1)
    FireStormWindowFinishBar:SetValue(1)
    FireStormWindowStatusBar:SetMinMax(0, 1)
    FireStormWindowStatusBar:SetValue(1)

    -- Cataclysm
    Medusa.Sunspire.SetBarSize('Cataclysm', Medusa.savedVariables.cataclysmWidth, Medusa.savedVariables.cataclysmHeight)
    Medusa.Sunspire.SetBarColor('Cataclysm', unpack(Medusa.savedVariables.cataclysmColor))
    Medusa.Sunspire.SetBarPosition('Cataclysm', Medusa.savedVariables.cataclysmWindowOffsetX, Medusa.savedVariables.cataclysmWindowOffsetY)
    CataclysmWindow:SetHidden(true)
    CataclysmWindowLabel:SetText(Medusa.Language.Events_Sunspire_Cataclysm)
    CataclysmWindowStatusBar:SetMinMax(0, 1)
    CataclysmWindowStatusBar:SetValue(1)

    -- Gravechill
    Medusa.Sunspire.SetBarSize('Gravechill', Medusa.savedVariables.gravechillWidth, Medusa.savedVariables.gravechillHeight)
    Medusa.Sunspire.SetBarColor('Gravechill', unpack(Medusa.savedVariables.gravechillColor))
    Medusa.Sunspire.SetBarPosition('Gravechill', Medusa.savedVariables.gravechillWindowOffsetX, Medusa.savedVariables.gravechillWindowOffsetY)
    GravechillWindow:SetHidden(true)
    GravechillWindowLabel:SetText(Medusa.Language.Events_Sunspire_Gravechill)
    GravechillWindowStatusBar:SetMinMax(0, 1)
    GravechillWindowStatusBar:SetValue(1)

    -- Time Shift
    Medusa.Sunspire.SetBarSize('TimeShift', Medusa.savedVariables.timeShiftWidth, Medusa.savedVariables.timeShiftHeight)
    Medusa.Sunspire.SetBarColor('TimeShift', unpack(Medusa.savedVariables.timeShiftColor))
    Medusa.Sunspire.SetBarPosition('TimeShift', Medusa.savedVariables.timeShiftWindowOffsetX, Medusa.savedVariables.timeShiftWindowOffsetY)
    TimeShiftWindow:SetHidden(true)
    TimeShiftWindowLabel:SetText(Medusa.Language.Events_Sunspire_TimeShift)
    TimeShiftWindowStatusBar:SetMinMax(0, 1)
    TimeShiftWindowStatusBar:SetValue(1)
end

---------------------------------------------------
-- Loads boss info  --
---------------------------------------------------
function Medusa.Sunspire.initBossData()
    -- Set name
    Medusa.Sunspire.boss.name = GetUnitName('boss1')

    -- Set id (based on name)
    local health = 0
    if Medusa.Sunspire.boss.name == Medusa.Language.Settings_Sunspire_Yolnahkriin then
        Medusa.Sunspire.boss.id = '1'
        health = Medusa.Sunspire.Yolnahkriin.health
    elseif Medusa.Sunspire.boss.name == Medusa.Language.Settings_Sunspire_Lokkestiiz then
        Medusa.Sunspire.boss.id = '2'
        health = Medusa.Sunspire.Lokkestiiz.health
    elseif Medusa.Sunspire.boss.name == Medusa.Language.Settings_Sunspire_Nahviintaas then
        Medusa.Sunspire.boss.id = '3'
        health = Medusa.Sunspire.Nahviintaas.health
    else
        Medusa.Sunspire.boss.id = '0'
    end

    -- Set difficulty. 1 = normal, 2 = veteran, 3 = hardmode
    Medusa.Sunspire.boss.difficulty = {}
    local difficulty = GetCurrentZoneDungeonDifficulty()
    if difficulty == 1 then
        Medusa.Sunspire.boss.difficultyId = 1
        Medusa.Sunspire.boss.difficultyName = 'Normal'
    elseif difficulty == 2 then
        -- Check boss health and adjust difficulty
        Medusa.Sunspire.boss.difficultyId = 2
        Medusa.Sunspire.boss.difficultyName = 'Veteran'

        local _activeHealth, max_health = GetUnitPower('reticleover', POWERTYPE_HEALTH)
        if health > 0 and health < max_health then
            Medusa.Sunspire.boss.difficultyId = 3
            Medusa.Sunspire.boss.difficultyName = 'Hardmode'
        end

        -- near boss, but boss is dead
        if _activeHealth < 1 then
            Medusa.Sunspire.boss.id = '0'
        end
    end
end

---------------------------------------------------
-- Sets new bar size  --
---------------------------------------------------
function Medusa.Sunspire.SetBarSize(_barName, _width, _height)
    local WindowFunction = _barName .. "Window"
    local WindowBackdropFunction = _barName .. "WindowBackdrop"
    local WindowStatusBarFunction = _barName .. "WindowStatusBar"
    local WindowLabelFunction = _barName .. "WindowLabel"
    local WindowLabelTimeFunction = _barName .. "WindowLabelTime"

    _G[WindowFunction]:SetDimensions(_width, _height)
    _G[WindowBackdropFunction]:SetDimensions(_width, _height)
    _G[WindowStatusBarFunction]:SetDimensions(_width, _height)
    _G[WindowLabelFunction]:SetDimensions(_width, _height)
    _G[WindowLabelTimeFunction]:SetDimensions(_width, _height)

    if _barName == "FireStorm" then
        FireStormWindowFinishBar:SetDimensions(_width / 2, _height)
    end
end

---------------------------------------------------
-- Sets new color  --
---------------------------------------------------
function Medusa.Sunspire.SetBarColor(_barName, r, g, b, a)
    local StatusBarFunction = _barName .. "WindowStatusBar"
    _G[StatusBarFunction]:SetColor(r, g, b, a)
end

---------------------------------------------------
-- Set bar to fully filled  --
---------------------------------------------------
function Medusa.Sunspire.ShowFullBar(_barName)
    local WindowStatusBarFunction = _barName .. "WindowStatusBar"
    _G[WindowStatusBarFunction]:SetMinMax(0, 1)
    _G[WindowStatusBarFunction]:SetValue(1)

    if _barName == "FireStorm" then
        FireStormWindowFinishBar:SetMinMax(0, 1)
        FireStormWindowFinishBar:SetValue(1)
    end
end

---------------------------------------------------
-- Saves bar location  --
---------------------------------------------------
function Medusa.Sunspire.SaveLoc(_barName)
    if _barName == "FocusedFire" then
        Medusa.savedVariables.focusedFireWindowOffsetX = FocusedFireWindow:GetLeft()
        Medusa.savedVariables.focusedFireWindowOffsetY = FocusedFireWindow:GetTop()
        Medusa.savedVariables.FocusedFireWindowLocation = TOPLEFT
    elseif _barName == "Emberstorm" then
        Medusa.savedVariables.emberstormWindowOffsetX = EmberstormWindow:GetLeft()
        Medusa.savedVariables.emberstormWindowOffsetY = EmberstormWindow:GetTop()
        Medusa.savedVariables.emberstormWindowLocation = TOPLEFT
    elseif _barName == "FireStorm" then
        Medusa.savedVariables.fireStormWindowOffsetX = FireStormWindow:GetLeft()
        Medusa.savedVariables.fireStormWindowOffsetY = FireStormWindow:GetTop()
        Medusa.savedVariables.fireStormWindowLocation = TOPLEFT
    elseif _barName == "Cataclysm" then
        Medusa.savedVariables.cataclysmWindowOffsetX = CataclysmWindow:GetLeft()
        Medusa.savedVariables.cataclysmWindowOffsetY = CataclysmWindow:GetTop()
        Medusa.savedVariables.cataclysmWindowLocation = TOPLEFT
    elseif _barName == "Gravechill" then
        Medusa.savedVariables.gravechillWindowOffsetX = GravechillWindow:GetLeft()
        Medusa.savedVariables.gravechillWindowOffsetY = GravechillWindow:GetTop()
        Medusa.savedVariables.gravechillWindowLocation = TOPLEFT
    elseif _barName == "TimeShift" then
        Medusa.savedVariables.timeShiftWindowOffsetX = TimeShiftWindow:GetLeft()
        Medusa.savedVariables.timeShiftWindowOffsetY = TimeShiftWindow:GetTop()
        Medusa.savedVariables.timeShiftWindowLocation = TOPLEFT
    end
end

---------------------------------------------------
-- Sets bar position  --
---------------------------------------------------
function Medusa.Sunspire.SetBarPosition(_barName, _offsetX, _offsetY)
    local WindowFunction = _barName .. "Window"
    _G[WindowFunction]:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, _offsetX, _offsetY)
end

---------------------------------------------------
-- Show/hide bars  --
---------------------------------------------------
function Medusa.Sunspire.ShowBars(_show)
    FireStormWindow:SetHidden(not _show)
    EmberstormWindow:SetHidden(not _show)
    FocusedFireWindow:SetHidden(not _show)
    FocusedFireWindowStatusBar:SetMinMax(0, 1)
    FocusedFireWindowStatusBar:SetValue(1)
    CataclysmWindow:SetHidden(not _show)
    GravechillWindow:SetHidden(not _show)
    TimeShiftWindow:SetHidden(not _show)
end

---------------------------------------------------
-- Lock/unlock bar movement  --
---------------------------------------------------
function Medusa.Sunspire.UnLockBars(_show)
    FireStormWindow:SetMovable(_show)
    EmberstormWindow:SetMovable(_show)
    FocusedFireWindow:SetMovable(_show)
    CataclysmWindow:SetMovable(_show)
    GravechillWindow:SetMovable(_show)
    TimeShiftWindow:SetMovable(_show)
end

---------------------------------------------------
-- Hides all windows and stops necessary events  --
---------------------------------------------------
function Medusa.Sunspire.Reset()
    -- Reset basic data
    Medusa.Sunspire.combatStart = 0
    Medusa.Sunspire.focusedFireStarted = 0
    Medusa.Sunspire.emberstormStarted = 0
    Medusa.Sunspire.fireStormStarted = 0
    Medusa.Sunspire.cataclysmStarted = 0
    Medusa.Sunspire.gravechillStarted = 0
    Medusa.Sunspire.abilities = {}

    -- Unregister combat events
    EVENT_MANAGER:UnregisterForEvent(Medusa.Sunspire.name, EVENT_COMBAT_EVENT)
    EVENT_MANAGER:UnregisterForUpdate("MedusaFocusedFire")
    EVENT_MANAGER:UnregisterForUpdate("MedusaEmberstorm")
    EVENT_MANAGER:UnregisterForUpdate("MedusaFireStorm")
    EVENT_MANAGER:UnregisterForUpdate("MedusaCataclysm")
    EVENT_MANAGER:UnregisterForUpdate("MedusaGravechill")
    EVENT_MANAGER:UnregisterForUpdate("TimeShiftGravechill")

    -- Hide bars
    FocusedFireWindow:SetHidden(true)
    EmberstormWindow:SetHidden(true)
    FireStormWindow:SetHidden(true)
    CataclysmWindow:SetHidden(true)
    GravechillWindow:SetHidden(true)
    TimeShiftWindow:SetHidden(true)
end